export const bk_tabs_header = "bk-tabs-header";
export const bk_headers_wrapper = "bk-headers-wrapper";
export const bk_headers = "bk-headers";
export const bk_tab = "bk-tab";
export const bk_close = "bk-close";
//# sourceMappingURL=tabs.js.map