export declare function union<T>(...sets: Set<T>[]): Set<T>;
export declare function intersection<T>(set: Set<T>, ...sets: Set<T>[]): Set<T>;
export declare function difference<T>(set: Set<T>, ...sets: Set<T>[]): Set<T>;
