export declare const bk_tool_icon_box_select = "bk-tool-icon-box-select";
export declare const bk_tool_icon_box_zoom = "bk-tool-icon-box-zoom";
export declare const bk_tool_icon_zoom_in = "bk-tool-icon-zoom-in";
export declare const bk_tool_icon_zoom_out = "bk-tool-icon-zoom-out";
export declare const bk_tool_icon_help = "bk-tool-icon-help";
export declare const bk_tool_icon_hover = "bk-tool-icon-hover";
export declare const bk_tool_icon_crosshair = "bk-tool-icon-crosshair";
export declare const bk_tool_icon_lasso_select = "bk-tool-icon-lasso-select";
export declare const bk_tool_icon_pan = "bk-tool-icon-pan";
export declare const bk_tool_icon_xpan = "bk-tool-icon-xpan";
export declare const bk_tool_icon_ypan = "bk-tool-icon-ypan";
export declare const bk_tool_icon_range = "bk-tool-icon-range";
export declare const bk_tool_icon_polygon_select = "bk-tool-icon-polygon-select";
export declare const bk_tool_icon_redo = "bk-tool-icon-redo";
export declare const bk_tool_icon_reset = "bk-tool-icon-reset";
export declare const bk_tool_icon_save = "bk-tool-icon-save";
export declare const bk_tool_icon_tap_select = "bk-tool-icon-tap-select";
export declare const bk_tool_icon_undo = "bk-tool-icon-undo";
export declare const bk_tool_icon_wheel_pan = "bk-tool-icon-wheel-pan";
export declare const bk_tool_icon_wheel_zoom = "bk-tool-icon-wheel-zoom";
export declare const bk_tool_icon_box_edit = "bk-tool-icon-box-edit";
export declare const bk_tool_icon_freehand_draw = "bk-tool-icon-freehand-draw";
export declare const bk_tool_icon_poly_draw = "bk-tool-icon-poly-draw";
export declare const bk_tool_icon_point_draw = "bk-tool-icon-point-draw";
export declare const bk_tool_icon_poly_edit = "bk-tool-icon-poly-edit";
export declare const bk_tool_icon_line_edit = "bk-tool-icon-line-edit";
